#include <sys/syscall.h>
#include <stdarg.h>
#include <sys/ioctl.h>

static int REAL_TIOCGETD = TIOCGETD;
#undef TIOCGETD
static int REAL_TIOCSETD = TIOCSETD;
#undef TIOCSETD
static int REAL_TIOCEXCL = TIOCEXCL;
#undef TIOCEXCL
static int REAL_TIOCNXCL = TIOCNXCL;
#undef TIOCNXCL
static int REAL_FIOCLEX = FIOCLEX;
#undef FIOCLEX
static int REAL_FIONCLEX = FIONCLEX;
#undef FIONCLEX

#include <termios.h>
#undef B0
#undef B110
#undef B1200
#undef B134
#undef B150
#undef B1800
#undef B200
#undef B2400
#undef B300
#undef B4800
#undef B50
#undef B600
#undef B75
#undef B9600
#undef BS0
#undef BS1
#undef CBRK
#undef CR0
#undef CR1
#undef CR2
#undef CR3
#undef ECHO
#undef EXTA
#undef EXTB
#undef FF0
#undef FF1
#undef NL0
#undef NL1
#undef TAB0
#undef TAB1
#undef TAB2
#undef XBREAK
#undef XTABS
#undef CEOT
#undef CERASE
#undef CINTR
#undef CKILL
#undef CSTART
#undef CSTOP


#include "../include/sgtty.h"
#include "../include/stdio.h"
#undef XTABS
#include "../include/sys/param.h"
#include "../include/sys/tty.h"

static int __linux_ioctl(int fd, int ioctl, void *arg) { return syscall(SYS_ioctl, fd, ioctl, arg); }

#define _linux_PARENB 0000400
#define _linux_PARODD 0001000
#define _linux_ICRNL  0000400
#define _linux_ECHO   0000010
#define _linux_IUCLC  0001000
#define _linux_TIOCEXCL	0x540C
#define _linux_TIOCNXCL	0x540D
#define _linux_TCFLSH		0x540B
#define _linux_FIONCLEX	0x5450
#define _linux_FIOCLEX	0x5451


int ioctl(int fd, unsigned long int request, ...)
{
	void *arg;
	va_list list;
	int two = 2;

	va_start(list, request);
	arg = va_arg(list, void *);

	switch(request) {
		case TIOCGETP:
			{
				struct termios t;
				struct ttiocb *o = (struct ttiocb *)arg;
				if (__linux_ioctl(fd, TCGETS, &t))
					return -1;
				o->ioc_ospeed = o->ioc_ispeed = 0;
				o->ioc_erase = t.c_cc[VERASE];
				o->ioc_kill = t.c_cc[VKILL];
				o->ioc_flags = 0;
				if (t.c_iflag & _linux_PARENB) {
					if (t.c_iflag & _linux_PARODD)
						o->ioc_flags |= ODDP;
					else
						o->ioc_flags |= EVENP;
				}
				if (t.c_iflag & _linux_ICRNL)
					o->ioc_flags |= CRMOD;
				if (t.c_iflag & _linux_ECHO)
					o->ioc_flags |= ECHO;
				if (t.c_iflag & _linux_IUCLC)
					o->ioc_flags |= LCASE;
				return 0;
			}
		case TIOCSETP:
		case TIOCSETN:
			fprintf(stderr, "v7 libc error: tiocset[pn] are unimplemented yet\n");
			break;
		case TIOCSETC:
			{
				struct termios t;
				struct tchars *c = (struct tchars *)c;

				if (__linux_ioctl(fd, TCGETS, &t))
					return -1;

				t.c_cc[VINTR] = c->t_intrc;
				t.c_cc[VQUIT] = c->t_quitc;
				t.c_cc[VSTART] = c->t_startc;
				t.c_cc[VSTOP] = c->t_stopc;
				t.c_cc[VEOF] = c->t_eofc;
				t.c_cc[VEOL] = c->t_brkc;

				if (__linux_ioctl(fd, TCSETS, &t))
					return -1;
			}
			break;
		case TIOCGETC:
			{
				struct termios t;
				struct tchars *c = (struct tchars *)c;

				if (__linux_ioctl(fd, TCGETS, &t))
					return -1;

				c->t_intrc = t.c_cc[VINTR];
				c->t_quitc = t.c_cc[VQUIT];
				c->t_startc = t.c_cc[VSTART];
				c->t_stopc = t.c_cc[VSTOP];
				c->t_eofc = t.c_cc[VEOF];
				c->t_brkc = t.c_cc[VEOL];
			}
			break;
		case TIOCEXCL: return __linux_ioctl(fd, _linux_TIOCEXCL, NULL);
		case TIOCNXCL: return __linux_ioctl(fd, _linux_TIOCNXCL, NULL);
		case FIOCLEX: return __linux_ioctl(fd, _linux_FIOCLEX, NULL);
		case FIONCLEX: return __linux_ioctl(fd, _linux_FIONCLEX, NULL);

		case TIOCFLUSH: return __linux_ioctl(fd, _linux_TCFLSH, &two);

		case TIOCHPCL:
		case MXLSTN:
		case MXNBLK:
			fprintf(stderr, "v7 libc error: unimplementable ioctl %d\n", request);
			break;

		case TIOCMODG:
		case TIOCMODS:
		case DIOCLSTN:
		case DIOCNTRL:
		case DIOCMPX:
		case DIOCNMPX:
		case DIOCSCALL:
		case DIOCRCALL:
		case DIOCPGRP:
		case DIOCGETP:
		case DIOCSETP:
		case DIOCLOSE:
		case DIOCTIME:
		case DIOCRESET:
			fprintf(stderr, "v7 libc error: unknown ioctl %d\n", request);
			break;


		default:
			fprintf(stderr, "v7 libc error: unknown ioctl request %d\n", request);
			abort();
	}

	va_end(list);


	return syscall(SYS_ioctl, fd, request, arg);
}

