#include <sys/syscall.h>

/* v7 system calls that translate direct linux system calls */
int write(int fd, const char *data, int len) { return syscall(SYS_write, fd, data, len); }
int close(int fd) { return syscall(SYS_close, fd); }
int read(int fd, char *buff, int count) { return syscall(SYS_read, fd, buff, count); }
int open(char *name, int mode) { return syscall(SYS_open, name, mode, 0); }
int creat(char *name, int mode) { return syscall(SYS_creat, name, mode); }
int lseek(int fd, int offs, int whence) { return syscall(SYS_lseek, fd, offs, whence); }
int kill(int pid, int sig) { return syscall(SYS_kill, pid, sig); }
int getpid(void) { return syscall(SYS_getpid); }
int time(int *t) { return syscall(SYS_time, t); }
int pipe(int *fds) { return syscall(SYS_pipe, fds); }
int fork(void) { return syscall(SYS_fork, 0); }
int wait(int *status) { return syscall(SYS_wait4, -1, status, 0, (void *)0); }
int alarm(int secs) { return syscall(SYS_alarm, secs); }
int pause(int secs) { return syscall(SYS_pause); }

int access(char *path, int mode) { return syscall(SYS_access, path, mode); }
int acct(char *path) { return syscall(SYS_acct, path); }
int chdir(char *path) { return syscall(SYS_chdir, path); }
int chroot(char *path) { return syscall(SYS_chroot, path); }
int chmod(char *path, int mode) { return syscall(SYS_chmod, path, mode); }
int chown(char *path, int o, int g) { return syscall(SYS_chown, o, g); }
int dup(int fd) { return syscall(SYS_dup, fd); }
int dup2(int fd1, int fd2) { return syscall(SYS_dup2, fd1, fd2); }
int getuid(void) { return syscall(SYS_getuid); }
int geteuid(void) { return syscall(SYS_geteuid); }
int getgid(void) { return syscall(SYS_getgid); }
int getegid(void) { return syscall(SYS_getegid); }
int setuid(int u) { return syscall(SYS_setuid, u); }
int setgid(int g) { return syscall(SYS_setuid, g); }
int link(char *p1, char *p2) { return syscall(SYS_link, p1, p2); }
int sync(void) { return syscall(SYS_sync); }
int unlink(char *path) { return syscall(SYS_unlink, path); }
int mknod(char *path, int m, int a) { return syscall(SYS_mknod, path, m, a); }
int mount(char *p1, char *p2, int rw) { return syscall(SYS_mount, p1, p2, rw); }
int umount(char *p1) { return syscall(SYS_umount2, p1, 0); }
int umask(int cm) { return syscall(SYS_umask, cm); }

#ifdef __NR_rt_sigprocmask
int _sigprocmask(int how, void *set, void *old) { return syscall(SYS_rt_sigprocmask, how, set, old, 128/8); }
#else
int _sigprocmask(int how, void *set, void *old) { return syscall(SYS_sigprocmask, how, set, old); }
#endif
void exit(int value);
void _exit(int value) { exit(value); }
int _v7libc_getdents(int fd, void *d, int c) { return syscall(SYS_getdents, fd, d, c); }

/* v7 system, calls that need some tweaks for us */
void exit(int value) {
	_cleanup();

	/* loop is to suppress gcc warning */
	while(1)
		syscall(SYS_exit, value);
}


extern void *__curbrk;
void *sbrk(int increment)
{
	void *old;
	if (__curbrk == (void *)0)
		if (brk((void *)0) < 0)
	    return (void *)-1;

	if (increment == 0)
		return __curbrk;

	old = __curbrk;
	if (brk (old + increment) < 0)
		return (void *) -1;

	return old;
}


void abort(void)
{
#define SIGIOT 6
	kill(getpid(), SIGIOT);
}




#include <stdarg.h>
static int _execle(int globenv, char *path, char *arg0, va_list list)
{
	extern char **environ;
	char **env;
	char *argv[1024];
	int n;

	argv[0] = arg0;
	for(n = 1; n < 1024; n++) {
		argv[n] = va_arg(list, char *);
		if (argv[n] == 0)
			break;
	}
	if (globenv)
		env = environ;
	else
		env = va_arg(list, char **);

	return  syscall(SYS_execve, path, argv, env);
}

int execle(char *path, char *arg, ...)
{
	int ret;
	va_list list;

	va_start(list, arg);

	ret = _execle(0, path, arg, list);
	va_end(list);
	return ret;
}

int execl(char *path, char *arg, ...)
{
	int ret;
	va_list list;

	va_start(list, arg);

	ret = _execle(1, path, arg, list);
	va_end(list);
	return ret;
}

int execv(char *path, char *argv[])
{
	extern char **environ;
	return syscall(SYS_execve, path, argv, environ);
}

int execve(char *path, char *argv[], char *env[])
{
	return syscall(SYS_execve, path, argv, env);
}



#define stat _linux_stat
#include "/usr/include/asm/stat.h"
#undef stat


#include "../include/sys/types.h"
#include "../include/sys/stat.h"

static void stat_copy(struct stat *buf, struct _linux_stat *st)
{
	buf->st_dev = st->st_dev;
	buf->st_ino = st->st_ino;
	buf->st_mode = st->st_mode & 0777777;
	buf->st_nlink = st->st_nlink;
	buf->st_uid = st->st_uid;
	buf->st_gid = st->st_gid;
	buf->st_rdev = st->st_rdev;
	buf->st_size = st->st_size;
	buf->st_atime = st->st_atime;
	buf->st_mtime = st->st_mtime;
	buf->st_ctime = st->st_ctime;

}

int stat(char *path, struct stat *buf)
{
	struct _linux_stat st;
	int ret;

	ret = syscall(SYS_stat, path, &st); 
	stat_copy(buf, &st);
	return ret;
}

int fstat(int fd, void *buf)
{
	struct _linux_stat st;
	int ret;

	ret = syscall(SYS_fstat, fd, &st);
	stat_copy(buf, &st);

	return ret;
}


int utime(char *path, int *t)
{
	struct utimbuf {
		long actime;
		long modtime;
	} ut;
	if (syscall(SYS_utime, path, &ut) != 0)
		return -1;
	t[0] = ut.actime;
	t[1] = ut.modtime;
	return 0;
}


#include "../include/sys/timeb.h"

int ftime(struct timeb *tb)
{
	struct timeval {
		int tv_sec;		/* seconds */
		int tv_usec;	/* microseconds */
	};

	struct timezone {
		int tz_minuteswest;	/* minutes west of Greenwich */
		int tz_dsttime;	/* type of dst correction */
	};

	struct timeval tv;
	struct timezone tz;
	syscall(SYS_gettimeofday, &tv, &tz);
	tb->time = tv.tv_sec;
	tb->millitm = (int)tv.tv_usec / (int)1000;
	tb->timezone = tz.tz_minuteswest;
	tb->dstflag = tz.tz_dsttime;

}

int nice(int incr)
{
	int pri;

	pri = syscall(SYS_getpriority, 0, 0);
	if (pri >= 0)
		pri = 20 - pri;

	pri += incr; /* no protection against overflow */
	if (syscall(SYS_setpriority, 0, 0, 20 - pri) < 0)
		return -1;

	return syscall(SYS_getpriority, 0, 0) == pri ? 0 : -1;
}
