/* Copyright (C) 1991, 1992, 1996, 1997, 2005 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.

	Heavily modified for v7libc by Tibor 'Igor2' Palinkas in 2011.
*/

#include <errno.h>
#include </usr/include/signal.h>
#include <sys/syscall.h>

/*
typedef void (*__sighandler_t)(int);
#define SIG_ERR ((void *)-1)
*/

#define HAVE_SA_RESTORER
/* This is the sigaction structure from the Linux 2.1.20 kernel.  */
struct old_kernel_sigaction {
    __sighandler_t k_sa_handler;
    unsigned long sa_mask;
    unsigned long sa_flags;
    void (*sa_restorer) (void);
};
/* This is the sigaction structure from the Linux 2.1.68 kernel.  */
struct kernel_sigaction {
    __sighandler_t k_sa_handler;
    unsigned long sa_flags;
    void (*sa_restorer) (void);
    __sigset_t sa_mask;
};


#ifndef __NR_sigaction
#define SIGACT __NR_rt_sigaction
#else
#define SIGACT __NR_sigaction
#endif

int sigaddset(__sigset_t *set, int signo);


/* Set the handler for the signal SIG to HANDLER,
   returning the old handler, or SIG_ERR on error.  */
__sighandler_t signal (int sig, __sighandler_t handler)
{
  struct kernel_sigaction act, oact;

  /* Check signal extents to protect __sigismember.  */
  if (handler == SIG_ERR || sig < 1 || sig >= NSIG)
    {
      __set_errno (EINVAL);
      return (void *)-1;
    }

  act.k_sa_handler = handler;
  if (__sigemptyset (&act.sa_mask) < 0)
    return (void *)-1;
  act.sa_flags = SA_ONESHOT | SA_NOMASK | SA_INTERRUPT;
  act.sa_flags &= ~SA_RESTART;
  if (syscall(SIGACT, sig, &act, &oact, _NSIG/8) < 0)
    return (void *)-1;

  return oact.k_sa_handler;
}



/* Code copied from dietlibc */
#define __sigmask(sig)		( ((unsigned long)1) << (((sig)-1) % (8*sizeof(unsigned long))) )
#define __sigword(sig)		( ((sig)-1) / (8*sizeof(unsigned long)) )

int sigaddset(__sigset_t *set, int signo)
{
	unsigned long __mask = __sigmask (signo);
	unsigned long __word = __sigword (signo);
	set->__val[__word]|=__mask;
	return 0;
}
